% Posterior mode rate estimation
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%   Thermodynamically Consistent Bayesian Analysis    %%%%%%%%%%%
%%%%%%%%%%%       of Closed Biochemical Reaction Systems        %%%%%%%%%%%
%%%%%%%%%%%            Posterior Mode Rate Estimation           %%%%%%%%%%%
%%%%%%%%%%%         Authors: Jenkinson, Zhong, Goutsias         %%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 11/04/2010              %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;              % clear all parameters
close all;              % close all files

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Input parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

I1 = 5000;              % number of first step SPSA iterations
L  = 50000;             % number of second step MA-MCMC iterations
I2 = 5000;              % number of third step SPSA iterations

stoich = xlsread('stoichmtry.xls'); % read stoichiometry matrix

time    = 1;            % simulation time (in min)
Q       = 6;            % number of measurement points
spacing = 0;            %  0 for logarithmic time point spacing
                        %  1 for uniform time point spacing 
sigma   = 0.3;          % standard deviation of measurement errors 

noiseType = 1;          % =1 for iid Gaussian noise
                        % =2 for correlated Gaussian noise
                        % =3 for iid uniformly distributed noise
                        
runWithPause = 0;       % =1 stop after each plot
                        % =0 run without stopping at each plot

x0_vec;                 % initial molecular concentrations
parameters;             % system parameter values

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Array initializations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

logkappa_init = repmat(0,[2*M,1]);
z             = repmat(0,[M,1]);
rmse          = repmat(0,[M+M1,1]);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Tolerances and options
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

svdtol      = 1e-8;               % singular value cut-off tolerance
tstep       = 0.005;              % time step for ODE integration
tol_high    = 1e-6;               % high tolerance for ODE solver
tol_low     = 1e-3;               % low tolerance for ODE solver
ndata       = time/tstep;         % data points for ODE solver
timings     = [0,time,ndata];     % passed to ODE solver
timingsFlux = [0,3*time,3*ndata]; % passed to ODE solver to find ss fluxes

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Generate time interval and sampling points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

timeint = floor(ndata/Q);

if spacing == 1
    tq = (timeint:timeint:ndata);
else
    tq = round(logspace(0.5,log10(ndata),Q));
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Set hyperparameter values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% Hyperparamaters of log-rate prior
%

logkappa_0 = -5.1010;    
tau        =  1.8990;  
lambda     =  0.7409;  

%
% Hyperparamaters of error variance prior
%

a = 3;                                                            
b = 1;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Perturbations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% 
% If pert(n) = 1, then the concentration of the nth species is perturbed. 
%

pert(1)  = 1;           %  Shc*
pert(2)  = 1;           %  Grb2
pert(3)  = 0;           %  Shc*-Grb2
pert(4)  = 1;           %  Sos
pert(5)  = 0;           %  Shc*-Grb2-Sos
pert(6)  = 0;           %  Grb2-Sos
pert(7)  = 0;           % (EGF-EGFR*)2-GAP
pert(8)  = 0;           % (EGF-EGFR*)2-GAP-Grb2
pert(9)  = 0;           % (EGF-EGFR*)2-GAP-Grb2-Sos
pert(10) = 0;           %  Ras-GDP
pert(11) = 0;           % (EGF-EGFR*)2-GAP-Grb2-Sos-Ras-GDP
pert(12) = 0;           %  Ras-GTP*
pert(13) = 0;           % (EGF-EGFR*)2-GAP-Grb2-Sos-Ras-GTP

P = sum(pert(1:N));     % total number of perturbations

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make unique foldername and generate new directory tree to save results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if spacing == 1
    FolderName = ['Q' num2str(Q) '_UniSample_Sigma' num2str(sigma,'%1.0e') '_P' num2str(P)];
else
    FolderName = ['Q' num2str(Q) '_LogSample_Sigma' num2str(sigma,'%1.0e') '_P' num2str(P)];
end

if noiseType == 1
    FolderName = [FolderName '_GausNoise'];
elseif noiseType == 2
    FolderName = [FolderName '_corGausNoise'];
elseif noiseType == 3
    FolderName = [FolderName '_UnifNoise'];
end

mkdir(['../results/' FolderName])

mkdir(['../results/' FolderName '/figures'])
mkdir(['../results/' FolderName '/reports'])

mkdir(['../results/' FolderName '/figures/comb'])
mkdir(['../results/' FolderName '/figures/credible'])
mkdir(['../results/' FolderName '/figures/data'])
mkdir(['../results/' FolderName '/figures/initial'])
mkdir(['../results/' FolderName '/figures/mean'])
mkdir(['../results/' FolderName '/figures/mode'])
mkdir(['../results/' FolderName '/figures/true'])
mkdir(['../results/' FolderName '/figures/flux'])

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Plot "true" vs. published dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

trueplotit(x0,kappaPublished,kappa,tol_high,timings,FolderName)

if runWithPause == 1
    pause;
end
close all

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Generate data for reference and perturbed systems. 
%%%%% The data are stored in files:
%%%%% data_0.txt - reference system 
%%%%% data_p.txt - perturbed systems
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[y,yp] = data(x0,x0p,pert,Q,sigma,tol_high,N,kappa,timings,tq,noiseType);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Plot dynamics and noisy measurements for reference system 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

plotit(x0,tstep,N,kappa,tol_high,timings,tq,FolderName);

if runWithPause == 1
    pause;
end
close all

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Estimate log-ratios of forward vs. reverse reaction rates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for m = 1:M;
    z(m) = log(kappa(2*(m-1)+1)/kappa(2*m));             % exact log-ratios
end

zest = zetaest(SS,permMatM,stoich,y,yp,Q,N,P,M,svdtol);  % estimated log-ratios

fprintf('\nLOG-RATIOS\n');
fprintf('----------\n');
fprintf('Reaction\tTrue\t\t Estimated\n');
for m = 1:M
    fprintf('%-1.0f\t\t   %8.4f\t\t%8.4f\n',m,z(m),zest(m))
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Plot true vs. initial dynamics for reference system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

priorMode = findPriorMode(logkappa_0,tau,lambda); 

for m = 1:M
    logkappa_init(2*m-1) = priorMode;
    logkappa_init(2*m)   = priorMode - zest(m);
end

kappa_init = exp(logkappa_init);

fprintf('\nREACTION RATE CONSTANTS\n');
fprintf('-----------------------\n');
fprintf('Reaction \t True values \t\t Initial values\n');
for m = 1:M
    fprintf('%+1.0f\t\t\t %-4.4e\t\t  %4.4e \t  \n',m,kappa(2*m-1),kappa_init(2*m-1))
    fprintf('%+1.0f\t\t\t %-4.4e\t\t  %4.4e \t  \n',-m,kappa(2*m),kappa_init(2*m))
end

tiplotit(x0,z,zest,kappa,kappa_init,tol_high,timings,FolderName);

if runWithPause == 1
    pause;
end

close all

%
% MEM - STEP 1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate posterior mode estimate starting at prior mode
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 

[logkappa_mode1,obj_kappa_mode1] = ...
    pmode(logkappa_init,M,N,I1,stoich,logkappa_0,tau,lambda,a,b,y,yp,...
          x0,x0p,pert,tq,tol_low,tol_high,svdtol,timings,W,permMatM);
      
%
% MEM - STEP 2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate posterior mean estimate by MA-MCMC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  

[logkappa_mean,logkappa_mode_MA,obj_kappa_mode_MA,internalKappa] = ...
    pmean(logkappa_mode1,M,N,L,stoich,logkappa_0,tau,lambda,a,b,y,yp,...
          x0,x0p,pert,tq,tol_high,svdtol,timings,W,permMatM);
    
kappa_mean = exp(logkappa_mean);

fprintf('\nREACTION RATE CONSTANTS\n');
fprintf('-----------------------\n');
fprintf('Reaction \t True values \t\t  Mean values\n');
for m = 1:M
    fprintf('%+1.0f\t\t\t %-4.4e\t\t  %4.4e \t  \n',m,kappa(2*m-1),kappa_mean(2*m-1))
    fprintf('%+1.0f\t\t\t %-4.4e\t\t  %4.4e \t  \n',-m,kappa(2*m),kappa_mean(2*m))
end
      
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Plot true vs. posterior mean dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tmnplotit(x0,tstep,N,kappa,kappa_mean,tol_high,timings,tq,FolderName);

if runWithPause == 1
    pause;
end

close all
                 
%
% MEM - STEP 3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate posterior mode estimate starting at posterior mean
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[logkappa_mode2,obj_kappa_mode2] = ...
    pmode(logkappa_mean,M,N,I2,stoich,logkappa_0,tau,lambda,a,b,y,yp,...
          x0,x0p,pert,tq,tol_low,tol_high,svdtol,timings,W,permMatM); 
                            
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Choose final mode estimator
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

modeEstimates = [logkappa_mode1,logkappa_mode_MA,logkappa_mode2];
objAtModeEsts = [obj_kappa_mode1,obj_kappa_mode_MA,obj_kappa_mode2]; 
logkappa_mode = modeEstimates(:,find(objAtModeEsts == max(max(objAtModeEsts)),1));
dlmwrite('../data/logmode.txt',logkappa_mode,'\t');

kappa_mode = exp(logkappa_mode);
[logkappa_mode,logKappaFreeMode] = imposeWcond(logkappa_mode,W,permMatM);

fprintf('\nREACTION RATE CONSTANTS\n');
fprintf('-----------------------\n');
fprintf('Reaction \t True values \t\t  Mode values\n');
for m = 1:M
    fprintf('%+1.0f\t\t\t %-4.4e\t\t  %4.4e \t  \n',m,kappa(2*m-1),kappa_mode(2*m-1))
    fprintf('%+1.0f\t\t\t %-4.4e\t\t  %4.4e \t  \n',-m,kappa(2*m),kappa_mode(2*m))
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Plot true vs. posterior mode dynamics for reference system 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tmdplotit(x0,tstep,N,kappa,kappa_mode,tol_high,timings,tq,FolderName);

if runWithPause == 1
    pause;
end

close all

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Plot all dynamics for reference system 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tallplotit(x0,tstep,N,kappa,kappa_mean,kappa_mode,tol_high,timings,...
           tq,FolderName,timingsFlux);

if runWithPause == 1
    pause;
end

close all

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Compute posterior covariance about mode 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

cv = repmat(0,[length(logKappaFreeMode),length(logKappaFreeMode)]);

for iter = 1:size(internalKappa,2)
    cv = cv + (internalKappa(:,iter)-logKappaFreeMode)...
             *(internalKappa(:,iter)-logKappaFreeMode)';
end

cv = cv./size(internalKappa,2);

clear internalKappa;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate posterior RMSE and D-optimal criterion
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for m = 1:length(logKappaFreeMode)
    rmse(m) = sqrt(cv(m,m));
end

%
% Apply SVD decomposition on covariance matrix
%

[U,S,U] = svd(cv); 

S0    = diag(S(S > svdtol*max(max(S))));
R0    = size(S0,1);
U0    = U(:,1:R0);
Uperp = U(:,(R0+1):end);
V0    = U0*sqrt(S0);

%
% Calculate D-optimal criterion
%

Dopt = log(det(S0))/R0;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate MED-AE AND MAX-AE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 

[medAE_mean,maxAE_mean] = trajerr(x0,x0p,pert,kappa,kappa_mean,tol_high,timings);
[medAE_mode,maxAE_mode] = trajerr(x0,x0p,pert,kappa,kappa_mode,tol_high,timings);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Write results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

writeit