% Wegscheider conditions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%   Thermodynamically Consistent Bayesian Analysis    %%%%%%%%%%%
%%%%%%%%%%%       of Closed Biochemical Reaction Systems        %%%%%%%%%%%
%%%%%%%%%%%            Posterior Mode Rate Estimation           %%%%%%%%%%%
%%%%%%%%%%%         Authors: Jenkinson, Zhong, Goutsias         %%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 11/04/2010              %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This function finds the Wegscheider conditions for a given stoichiometry 
% matrix by returning the following variables:
%
% permMatM - An MxM permutation matrix that converts the given 
%            stoichiometry matrix into a matrix with the first M1 columns
%            being linearly independent according to the following equation
%            stoichM = stoich*permMatM;
%            note that stoich = stoichM*permMatM';
% 
% permMatN - An NxN permutation matrix that converts a stoichiometry 
%            matrix into a matrix with the first M1 rows being linearly 
%            independent according to the following equation
%            stoichNew = permMatN*stoichM;
%            note that stoichM = permMatN'*stoichNew;
%
% SS       - The M1xM2 matrix inv(S11)*S12
%
% W        - The (M-M2)x(M+M1) matrix which gives the "dependent" variables 
%            from the "free" variables according to kd = W*kf; 
%            kf in this case is the log-rate constants constructed as follows:
%
%            kf = [newLogKapForward;newLogKapBackward(1:M1)];
%     
%            where 
%
%            newLogKapForward'  = logKappaForward'*permMatM;
%            newLogKapBackward' = logKappaBackward'*permMatM;
%

function [SS,W,permMatN,permMatM,M1,M2] = Wcond(stoich)

%
% find linearly independent columns
%

[R,licols] = rref(stoich);

%
% Set parameters
%

N  = size(stoich,1);
M  = size(stoich,2);
M1 = length(licols); % rank(stoich)
M2 = M-M1;

%
% find linearly dependent columns
%

ldcols = [];
for m = 1:M
    if sum(licols==m) == 0
        ldcols = [ldcols m];
    end
end

%
% generate first permutation matrix
%

permMatM = eye(M);
permMatM = [permMatM(:,licols),permMatM(:,ldcols)];

stoichM = stoich*permMatM;

%
% find linearly independent rows
%

[R,lirows] = rref(stoichM(:,1:length(licols))');

%
% find linearly dependent rows
%

ldrows = [];
for n = 1:N
    if sum(lirows==n)==0
        ldrows = [ldrows n];
    end
end

%
% generate second permutation matrix
%

permMatN = eye(N);
permMatN = [permMatN(lirows,:);permMatN(ldrows,:)];

%
% Final stoichiometery matrix in block form
%

stoichNew = permMatN*stoichM;

S11 = stoichNew(1:M1,1:M1);
S12 = stoichNew(1:M1,(M1+1):end);

%
% Compute SS & W matrices
%

SS = (inv(S11)*S12)';
W  = [-SS eye(M2) SS];