% Data generation
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%   Thermodynamically Consistent Bayesian Analysis    %%%%%%%%%%%
%%%%%%%%%%%       of Closed Biochemical Reaction Systems        %%%%%%%%%%%
%%%%%%%%%%%            Posterior Mode Rate Estimation           %%%%%%%%%%%
%%%%%%%%%%%         Authors: Jenkinson, Zhong, Goutsias         %%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 11/04/2010              %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function [y,yp] = data(x0,x0p,pert,Q,sigma,tol_high,N,kappa,timings,...
                       tq,noiseType)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initializations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

P  = sum(pert(1:N));        % number of perturbations
yp = repmat(0,[P*(Q+1),N]);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Generate measurements for reference system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[t,x] = oden13(kappa,x0,[tol_high;tol_high],timings);

%
% Generate Noise
%

if noiseType == 1 
    % iid Gaussian noise
    eta = sigma.*randn(Q+1,N);
    
elseif noiseType == 2 
    % correlated Gaussian noise
    CovMat=zeros(Q+1,Q+1);
    for index1 = 1:Q
        for index2 = 1:Q
            CovMat(index1,index2)=(sigma^2)*exp(-abs(t(tq(index1))-t(tq(index2))));
        end
    end
    CovMat(Q+1,Q+1) = sigma^2;
    
    C         = chol(CovMat);
    iidNoise  = randn(Q+1,N);
    eta       = C'*iidNoise;
    
elseif noiseType == 3 
    % iid uniformly distributed noise
    intervalLength = sigma*sqrt(12);
    eta            = intervalLength*rand(Q+1,N)-0.5*intervalLength;
    
end

%
% Make noisy data
%

y          = log(abs(x(tq,1:N))) + eta(1:Q,1:N);
y(Q+1,1:N) = log(abs(x(tq(Q),1:N))) + eta(Q+1,1:N);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Generate measurements for perturbed systems
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

index = 0;

for p = 1:N
    if pert(p) == 1
        index = index+1;
        [t,xtmp] = oden13(kappa,x0p(:,p),[tol_high;tol_high],timings);
        
        %
        % Generate Noise
        %
        
        if noiseType == 1 
            % iid Gaussian noise
            eta = sigma.*randn(Q+1,N);

        elseif noiseType == 2 
            % correlated Guassian noise
            CovMat=zeros(Q+1,Q+1);
            for index1 = 1:Q
                for index2 = 1:Q
                    CovMat(index1,index2)=(sigma^2)*exp(-abs(t(tq(index1))-t(tq(index2))));
                end
            end
            CovMat(Q+1,Q+1) = sigma^2;

            C         = chol(CovMat);
            iidNoise  = randn(Q+1,N);
            eta       = C'*iidNoise;

        elseif noiseType == 3 
            % iid uniformly distributed noise
            intervalLength = sigma*sqrt(12);
            eta            = intervalLength*rand(Q+1,N)-.5*intervalLength;

        end
        yp(((index-1)*Q+1):(index*Q),1:N) = log(abs(xtmp(tq,1:N)))    + eta(1:Q,1:N);
        yp(P*Q+index,1:N)                 = log(abs(xtmp(tq(Q),1:N))) + eta(Q+1,1:N);
    end   
end

dlmwrite('../data/data_0.txt',y,'\t');
dlmwrite('../data/data_p.txt',yp,'\t');