% Objective function evaluation
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%   Thermodynamically Consistent Bayesian Analysis    %%%%%%%%%%%
%%%%%%%%%%%       of Closed Biochemical Reaction Systems        %%%%%%%%%%%
%%%%%%%%%%%            Posterior Mode Rate Estimation           %%%%%%%%%%%
%%%%%%%%%%%         Authors: Jenkinson, Zhong, Goutsias         %%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 11/04/2010              %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function obj = object(y,yp,x0,x0p,pert,kappahat,N,Xi,ytilde,logkappa_0,...
                      lambda,tau,a,b,tolerance,timings,tq,W,permMatM)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

Q        = size(y,1)-1;    % number of time points   
P        = sum(pert(1:N)); % number of perturbations

beta  = a + N*Q*(P+1)/2;

%
% Find full kappahat if kappahat is only the free variables
%

if length(kappahat)<2*size(permMatM,1)
    [logKappaFeas,logKappaFree] = imposeWcond(kappahat,W,permMatM);
end

kappaexp = exp(logKappaFeas);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate phi
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[t,xtmp] = oden13(kappaexp,x0,tolerance,timings);

diff = y(1:Q,1:N) - log(abs(xtmp(tq,1:N)));
phi  = sum(sum(diff.*diff));      % for reference system

tag   = zeros(1,N);
index = 0;

for i = 1:N
    if pert(i) == 1
        index  = index + 1;
        tag(i) = index;
    end
end

for p = 1:N
    if pert(p) > 0
        [t,xtmp] = oden13(kappaexp,x0p(:,p),tolerance,timings);
        diff = yp((tag(p)-1)*Q+1:tag(p)*Q,:)-log(abs(xtmp(tq,1:N)));
        phip = sum(sum(diff.*diff));  % for perturbed systems
        phi  = phi + phip;            % for reference and perturbed systems
    end
end

phi = 2*b + phi;                % total phi

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate psi
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

psi = 2*b/(P+1) + (logKappaFeas([1,3,5,7,9,11,13,15,17])...
                   -logKappaFeas([2,4,6,8,10,12,14,16,18])-ytilde)'*Xi*...
                   (logKappaFeas([1,3,5,7,9,11,13,15,17]) ...
                   -logKappaFeas([2,4,6,8,10,12,14,16,18])-ytilde);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate forward rate prior term
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

prior = 0;

for m = [1 3 5 7 9 11 13 15 17]
    prior = prior + logKappaFeas(m)/tau + log(erfc((lambda/tau + ...
                   (logKappaFeas(m)-logkappa_0)/lambda)/sqrt(2)));
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate objective function value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

obj = prior - a*log(psi) - beta*log(phi);