% Plot true vs. posterior mode molecular dynamics
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%   Thermodynamically Consistent Bayesian Analysis    %%%%%%%%%%%
%%%%%%%%%%%       of Closed Biochemical Reaction Systems        %%%%%%%%%%%
%%%%%%%%%%%            Posterior Mode Rate Estimation           %%%%%%%%%%%
%%%%%%%%%%%         Authors: Jenkinson, Zhong, Goutsias         %%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 11/04/2010              %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function tallplotit(x0,tstep,N,kappa,kappa_mean,kappa_mode,tol_high,...
                    timings,tq,FolderName,timingsFlux)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

close all

tmp = dlmread('../data/data_0.txt');
Q = size(tmp,1)-1;
ytmp(1:Q,1:N) = tmp(1:Q,1:N);
ytmp = exp(ytmp);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate true and posterior mode estimated molecular dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[t1,init]  = oden13(kappa,x0,[tol_high;tol_high],timings);
[t1,xmean] = oden13(kappa_mean,x0,[tol_high;tol_high],timings);
[t1,xmode] = oden13(kappa_mode,x0,[tol_high;tol_high],timings);
[t2,xflux] = oden13(kappa_mode,x0,[tol_high;tol_high],timingsFlux);

%
% Calculate fluxes at posterior mode
%

feasFlux = findFluxes(xflux,kappa_mode);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Make plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

set(0,'DefaultAxesLineWidth',2,'DefaultAxesFontSize',14,...
      'DefaultAxesFontWeight','normal');
lnw = 2;

figure
plot([1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9],log(kappa),'rx',...
     'MarkerSize',11,'Color','r','LineWidth',2)
hold on
plot([1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9],log(kappa_mean),'o',...
     'MarkerSize',11,'Color','g','LineWidth',2)
hold on
plot([1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9],log(kappa_mode),'square',...
     'MarkerSize',11,'Color','b','LineWidth',2)
axis tight
xlabel('\bf reaction','fontsize',12);
ylabel('\bf log-rate value','fontsize',12);
title('true (red), mean (green), and mode (blue) reaction rates')
saveas(gcf,['../results/' FolderName '/figures/comb/comb_0.fig'])

figure
plot(tq*tstep,ytmp(:,1),'k*');
hold on
plot(t1,init(:,1),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,1),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,1),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_1:  Shc*}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_1.fig'])
   
figure
plot(tq*tstep,ytmp(:,2),'k*');
hold on
plot(t1,init(:,2),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,2),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,2),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_2:  Grb2}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_2.fig'])

figure
plot(tq*tstep,ytmp(:,3),'k*');
hold on
plot(t1,init(:,3),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,3),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,3),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_3:  Shc*-Grb2}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_3.fig'])
   
figure
plot(tq*tstep,ytmp(:,4),'k*');
hold on
plot(t1,init(:,4),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,4),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,4),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_4:  Sos}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_4.fig'])

figure
plot(tq*tstep,ytmp(:,5),'k*');
hold on
plot(t1,init(:,5),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,5),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,5),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_5:  Shc*-Grb2-Sos}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_5.fig'])
   
figure
plot(tq*tstep,ytmp(:,6),'k*');
hold on
plot(t1,init(:,6),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,6),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,6),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_6:  Grb2-Sos}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_6.fig'])

figure
plot(tq*tstep,ytmp(:,7),'k*');
hold on
plot(t1,init(:,7),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,7),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,7),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_7:  (EGF-EGFk*)2-GAP}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_7.fig'])

figure
plot(tq*tstep,ytmp(:,8),'k*');
hold on
plot(t1,init(:,8),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,8),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,8),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_8:  (EGF-EGFk*)2-GAP-Grb2}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_8.fig'])

figure
plot(tq*tstep,ytmp(:,9),'k*');
hold on
plot(t1,init(:,9),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,9),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,9),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_9:  (EGF-EGFk*)2-GAP-Grb2-Sos}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_9.fig'])

figure
plot(tq*tstep,ytmp(:,10),'k*');
hold on
plot(t1,init(:,10),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,10),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,10),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_{10}:  Ras-GDP}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_10.fig'])

figure
plot(tq*tstep,ytmp(:,11),'k*');
hold on
plot(t1,init(:,11),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,11),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,11),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_{11}:  (EGF-EGFk*)2-GAP-Grb2-Sos-Ras-GDP}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_11.fig'])

figure
plot(tq*tstep,ytmp(:,12),'k*');
hold on
plot(t1,init(:,12),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,12),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,12),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_{12}:  Ras-GTP*}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_12.fig'])

figure
plot(tq*tstep,ytmp(:,13),'k*');
hold on
plot(t1,init(:,13),'Color',[1 0 0],'Linewidth',lnw)
hold on
plot(t1,xmean(:,13),'Color',[0 1 0],'Linewidth',lnw)
hold on
plot(t1,xmode(:,13),'Color',[0 0 1],'Linewidth',lnw)
h = legend(' data',' true',' mean',' mode');
set(h,'Location','NorthEast');
   xlabel('time (minutes)');
   ylabel('molecules/cell');
   title('{\bf X_{13}:  (EGF-EGFk*)2-GAP-Grb2-Sos-Ras-GTP}');
saveas(gcf,['../results/' FolderName '/figures/comb/comb_13.fig'])




figure;
plot(t2,feasFlux,'LineWidth',lnw)
xlabel('time (minutes)')
ylabel('flux (molecules/minute)')
title('Fluxes at Mode')
saveas(gcf,['../results/' FolderName '/figures/flux/fluxes.fig'])